@@ -136,6 +136,9 @@ _playlist_item_metadata_cb(MafwPlaylist *pls,
   GValue* value;
   const gchar* title;
   value = mafw_metadata_first(metadata, MAFW_METADATA_KEY_TITLE);
+  if(value == NULL)
+    return;
+
   title = g_value_get_string(value);
 
   MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
@@ -148,8 +151,8 @@ _playlist_item_metadata_cb(MafwPlaylist *pls,
     desktop_plugin->priv->title = g_strdup(title);
   }
   value = mafw_metadata_first(metadata, MAFW_METADATA_KEY_ARTIST);
-  title = g_value_get_string(value);
-  if(title != NULL)
+
+  if(G_VALUE_HOLDS_STRING(value) && (title = g_value_get_string(value)) != NULL)
   {
     if(desktop_plugin->priv->artist)
     {
@@ -158,8 +161,7 @@ _playlist_item_metadata_cb(MafwPlaylist *pls,
     desktop_plugin->priv->artist = g_strdup(title);
   }
   value = mafw_metadata_first(metadata, MAFW_METADATA_KEY_ALBUM);
-  title = g_value_get_string(value);
-  if(title != NULL)
+  if(G_VALUE_HOLDS_STRING(value) && (title = g_value_get_string(value)) != NULL)
   {
     if(desktop_plugin->priv->album)
     {
@@ -239,6 +241,7 @@ _on_renderer_get_status_cb(MafwRenderer* mafw_renderer,
 static void
 _on_media_changed_cb(MafwRenderer* mafw_renderer, gint index, gchar* object_id, gpointer user_data)
 {
+  g_warning("on media changed\n");
   MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
   mafw_renderer_get_status(desktop_plugin->priv->mafw_renderer,
 			   _on_renderer_get_status_cb, 
@@ -256,6 +259,22 @@ _on_state_changed_cb(MafwRenderer* mafw_renderer, gint state, gpointer user_data
 static void
 _on_metadata_changed_cb(MafwRenderer* mafw_renderer, gchar *name, GValueArray *value, gpointer user_data)
 {
+  /* 
+  if(value->n_values == 1)
+  {
+    GValue* v;
+    v = g_value_array_get_nth(value, 0);
+    if(v && G_VALUE_HOLDS_STRING(v))
+    {
+      const gchar* value_str = g_value_get_string(v);
+      g_warning("value %s\n", value_str);
+    }
+  }
+  */
+  MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
+  mafw_renderer_get_status(desktop_plugin->priv->mafw_renderer,
+			   _on_renderer_get_status_cb, 
+			   desktop_plugin);
 }
 
 static void
@@ -540,7 +559,6 @@ _on_playback_state_changed(MafwRenderer* renderer,
 			   gpointer user_data,
 			   const GError* error)
 {
-  g_warning("playback state changed\n");
   if(error)
   {
     g_warning("error on playbackstate cb %s\n", error->message);
@@ -616,46 +634,100 @@ _on_play_pause_button_press(GtkWidget* widget, GdkEventButton* event, gpointer u
   }
 }
 
-static void
-_on_label_button_press(GtkWidget* widget, GdkEventButton* event, gpointer user_data)
+typedef struct _playlist_selection_data_t
 {
-  MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
-  GtkWidget* dialog = hildon_picker_dialog_new(GTK_WINDOW(desktop_plugin));
-  GtkWidget* selector = hildon_touch_selector_new_text();
-  hildon_touch_selector_set_column_selection_mode(HILDON_TOUCH_SELECTOR(selector),
-						  HILDON_TOUCH_SELECTOR_SELECTION_MODE_SINGLE);
-  desktop_plugin->priv->mafw_playlist_manager = mafw_playlist_manager_get();
-  GArray* playlists = mafw_playlist_manager_list_playlists(desktop_plugin->priv->mafw_playlist_manager, NULL);
-  int i = 0;
-  for(i = 0; i< playlists->len;++i)
-  {
-    MafwPlaylistManagerItem* item = &g_array_index(playlists, MafwPlaylistManagerItem, i);
+  GtkWidget* dialog;
+  MediaPlayerWidgetHomePlugin* desktop_plugin;
+  GtkListStore* playlist_store;
+} playlist_selection_data_t;
 
-    if(g_strstr_len(item->name, 3, "Fmp") == NULL)
+static void
+_on_playlist_dialog_response(GtkDialog* dialog,
+			     gint response_id,
+			     gpointer user_data)
+{
+  playlist_selection_data_t* playlist_selection_data = (playlist_selection_data_t*)user_data;
+  gtk_widget_hide(GTK_WIDGET(dialog));
+  gtk_widget_destroy(playlist_selection_data->dialog);
+  g_object_unref(playlist_selection_data->playlist_store);
+  g_free(playlist_selection_data);
+}
+
+static void
+_playlist_item_request_cb(MafwSource *source,
+			  guint browse_id,
+			  gint remaining,
+			  guint index,
+			  const gchar *object_id,
+			  GHashTable *metadata,
+			  gpointer user_data,
+			  const GError *error)
+{
+  MafwPlaylist* playlist = MAFW_PLAYLIST(user_data);
+  if(object_id != NULL) 
+  {
+    mafw_playlist_append_item(playlist,
+			      object_id,
+			      NULL);
+  }
+}
+
+static MafwSource*
+_get_mafw_source(MafwRegistry* mafw_registry)
+{
+  GList* sources = mafw_registry_get_sources(mafw_registry);
+  GList* source = sources;
+  while(source)
+  {
+    MafwSource* mafw_source = MAFW_SOURCE(source->data);
+    if(g_strcmp0(mafw_extension_get_name(MAFW_EXTENSION(mafw_source)), MEDIAPLAYER_SOURCE) == 0)
     {
-      hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(selector),
-					item->name);
+      return mafw_source;
+    }
+    else
+    {
+      source = source->next;
     }
   }
-  hildon_touch_selector_set_active(HILDON_TOUCH_SELECTOR(selector),
-				   0, -1);
-  hildon_picker_dialog_set_selector(HILDON_PICKER_DIALOG(dialog), 
-				    HILDON_TOUCH_SELECTOR(selector));
-  gtk_widget_show_all(dialog);
-  MafwProxyPlaylist* new_playlist = NULL;
-  int response = gtk_dialog_run(GTK_DIALOG(dialog));
-  if(response == GTK_RESPONSE_OK)
+  return NULL;
+}
+
+static void
+_on_playlist_item_selected(HildonTouchSelector* selector, 
+			   gint column,
+			   gpointer user_data)
+{
+  playlist_selection_data_t* playlist_selection_data = (playlist_selection_data_t*)user_data;
+  GtkTreeIter iter;
+  hildon_touch_selector_get_selected(HILDON_TOUCH_SELECTOR(selector),
+				     0, &iter);
+  gchar* playlist_name = NULL;
+  gchar* playlist_oid = NULL;
+  if(gtk_list_store_iter_is_valid(playlist_selection_data->playlist_store, &iter))
   {
-    gint index = hildon_touch_selector_get_active(HILDON_TOUCH_SELECTOR(selector), 0);
-    gchar* selected = hildon_touch_selector_get_current_text(HILDON_TOUCH_SELECTOR(selector));
-    new_playlist= mafw_playlist_manager_create_playlist(desktop_plugin->priv->mafw_playlist_manager, 
-							selected,
-							NULL);
-      
-    MafwProxyPlaylist* old_playlist= mafw_playlist_manager_create_playlist(desktop_plugin->priv->mafw_playlist_manager, 
-									   "FmpAudioPlaylist",
-									   NULL);
-      
+    gint id = 0;
+    gtk_tree_model_get(GTK_TREE_MODEL(playlist_selection_data->playlist_store),
+		       &iter, 
+		       0, &playlist_name,
+		       1, &playlist_oid,
+		       -1);
+  }
+
+  MafwPlaylistManager* mafw_playlist_manager = mafw_playlist_manager_get();
+  MafwProxyPlaylist* old_playlist = mafw_playlist_manager_create_playlist(mafw_playlist_manager, 
+									  "FmpAudioPlaylist",
+									  NULL);
+
+  mafw_renderer_pause(playlist_selection_data->desktop_plugin->priv->mafw_renderer,
+		      NULL, NULL);
+  
+  if(g_strcmp0(playlist_oid, "intern") == 0)
+  {
+    MafwProxyPlaylist* new_playlist = NULL;
+    new_playlist = mafw_playlist_manager_create_playlist(mafw_playlist_manager, 
+							 playlist_name,
+							 NULL);
+    g_free(playlist_name);
       
     if(mafw_playlist_clear(MAFW_PLAYLIST(old_playlist),
 			   NULL))
@@ -681,15 +753,180 @@ _on_label_button_press(GtkWidget* widget, GdkEventButton* event, gpointer user_d
       }
       g_strfreev(items);
     }
-    if(mafw_renderer_assign_playlist(desktop_plugin->priv->mafw_renderer,
+    if(mafw_renderer_assign_playlist(playlist_selection_data->desktop_plugin->priv->mafw_renderer,
 				     MAFW_PLAYLIST(old_playlist),
 				     NULL))
-    {}
+    {
+      mafw_renderer_play(playlist_selection_data->desktop_plugin->priv->mafw_renderer,
+			 _on_playback_state_changed, playlist_selection_data->desktop_plugin);
+    }
   }
-  mafw_playlist_manager_free_list_of_playlists(playlists);
-  gtk_widget_destroy(dialog);
-  mafw_renderer_play(desktop_plugin->priv->mafw_renderer,
-		     _on_playback_state_changed, desktop_plugin);
+  else
+  {
+    MafwSource* mafw_source = _get_mafw_source(playlist_selection_data->desktop_plugin->priv->mafw_registry);
+    if(mafw_source)
+    {
+      const gchar *const *keys;
+      keys = MAFW_SOURCE_LIST(MAFW_METADATA_KEY_TITLE,
+			      MAFW_METADATA_KEY_ARTIST,
+			      MAFW_METADATA_KEY_ALBUM,
+			      MAFW_METADATA_KEY_ALBUM_ART_SMALL_URI);
+
+      mafw_playlist_clear(MAFW_PLAYLIST(old_playlist), NULL);
+      mafw_source_browse(mafw_source,
+			 playlist_oid,
+			 FALSE,
+			 NULL,
+			 NULL,
+			 keys,
+			 0, MAFW_SOURCE_BROWSE_ALL,
+			 _playlist_item_request_cb,
+			 old_playlist);
+    }
+  }
+  gtk_dialog_response(GTK_DIALOG(playlist_selection_data->dialog), GTK_RESPONSE_OK);
+}
+
+static void
+_browse_playlists_cb(MafwSource *source,
+		     guint browse_id,
+		     gint remaining,
+		     guint index,
+		     const gchar *object_id,
+		     GHashTable *metadata,
+		     gpointer user_data,
+		     const GError *error)
+{
+  playlist_selection_data_t* playlist_selection_data = (playlist_selection_data_t*)user_data;
+  if(remaining == 0)
+  {
+    /*
+    hildon_gtk_window_set_progress_indicator(GTK_WINDOW(playlist_selection_data->dialog),
+					     0);
+    */
+  }
+
+  if(object_id != NULL) 
+  {
+    if(metadata != NULL) 
+    {
+      const gchar *playlist_name;
+      GValue *value;
+      value = mafw_metadata_first (metadata,
+				   MAFW_METADATA_KEY_TITLE);
+      if(value)
+      {
+	playlist_name = g_value_get_string(value);
+	GtkTreeIter iter;
+	gtk_list_store_append(playlist_selection_data->playlist_store, &iter);
+	gtk_list_store_set(playlist_selection_data->playlist_store, &iter,
+			   0, playlist_name,
+			   1, object_id,
+			   -1);
+      }
+    }
+  }
+}
+ 
+static void
+_do_playlist_request(MafwSource* mafw_source, playlist_selection_data_t* playlist_selection_data)
+{
+  const gchar *const *keys;
+  gchar *object_id = "localtagfs::music/playlists";
+  
+  keys = MAFW_SOURCE_LIST(MAFW_METADATA_KEY_TITLE);
+  mafw_source_browse(mafw_source,
+		     object_id,         
+		     FALSE,             
+		     NULL,              
+		     NULL,              
+		     keys,              
+		     0, 100,             
+		     _browse_playlists_cb,
+		     playlist_selection_data);    
+}
+
+static void
+_fill_playlist_list_from_manager(GtkListStore* playlist_store)
+{
+  MafwPlaylistManager* mafw_playlist_manager = mafw_playlist_manager_get();
+  if(mafw_playlist_manager)
+  {
+    GArray* playlists = mafw_playlist_manager_list_playlists(mafw_playlist_manager, NULL);
+    int i = 0;
+    GtkTreeIter iter;
+    for(i = 0; i< playlists->len;++i)
+    {
+      MafwPlaylistManagerItem* item = &g_array_index(playlists, MafwPlaylistManagerItem, i);
+      
+      if(g_strstr_len(item->name, 3, "Fmp") == NULL)
+      {
+	gtk_list_store_append(playlist_store, &iter);
+	gtk_list_store_set(playlist_store, &iter,
+			   0, item->name,
+			   1, "intern",
+			   -1);
+      }
+    }
+    mafw_playlist_manager_free_list_of_playlists(playlists);
+  }
+}
+
+static void
+_fill_playlist_list_from_mafw_source(playlist_selection_data_t* playlist_selection_data, MafwRegistry* mafw_registry)
+{
+  MafwSource* mafw_source = _get_mafw_source(mafw_registry);
+  if(mafw_source)
+  {
+    _do_playlist_request(mafw_source, playlist_selection_data);
+  }
+  else
+  {
+    /*
+    hildon_gtk_window_set_progress_indicator(GTK_WINDOW(playlist_selection_data->dialog),
+					     0);
+    */
+  }
+}
+
+static void
+_on_label_button_press(GtkWidget* widget, GdkEventButton* event, gpointer user_data)
+{
+  MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
+  GtkListStore* playlist_store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+
+  GtkWidget* dialog = hildon_dialog_new();
+  gtk_window_set_title(GTK_WINDOW(dialog), "Playlist");
+  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
+  gtk_window_set_default_size(GTK_WINDOW(dialog), -1, 330);
+  
+  GtkWidget* selector = hildon_touch_selector_new_text();
+  hildon_touch_selector_set_column_selection_mode(HILDON_TOUCH_SELECTOR(selector),
+						  HILDON_TOUCH_SELECTOR_SELECTION_MODE_SINGLE);
+  hildon_touch_selector_set_model(HILDON_TOUCH_SELECTOR(selector),
+				  0, GTK_TREE_MODEL(playlist_store));
+
+  desktop_plugin->priv->mafw_playlist_manager = mafw_playlist_manager_get();
+
+  playlist_selection_data_t* playlist_selection_data = g_new0(playlist_selection_data_t, 1);
+  playlist_selection_data->dialog = dialog;
+  playlist_selection_data->desktop_plugin = desktop_plugin;
+  playlist_selection_data->playlist_store = playlist_store;
+  /*
+  hildon_gtk_window_set_progress_indicator(GTK_WINDOW(dialog),
+					   1);
+  */
+  _fill_playlist_list_from_manager(playlist_store);
+  _fill_playlist_list_from_mafw_source(playlist_selection_data, desktop_plugin->priv->mafw_registry);
+
+  hildon_touch_selector_set_active(HILDON_TOUCH_SELECTOR(selector),
+				   0, -1);
+  
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), selector, TRUE, TRUE, 0);
+  g_signal_connect(selector, "changed", G_CALLBACK(_on_playlist_item_selected), playlist_selection_data);
+  g_signal_connect(dialog, "response", G_CALLBACK(_on_playlist_dialog_response), playlist_selection_data);
+  gtk_widget_show_all(dialog);
+  gtk_dialog_run(GTK_DIALOG(dialog));
 }
 
 static gboolean
